"use strict";
var EXPIRE, Promise, cache, latestVer, resolveVerTag, strategy;
cache = require('./cache');
strategy = require('./strategy');
latestVer = require('./latestVersion');
EXPIRE = 20 * 60 * 1000;
Promise = require('nofs').Promise;
resolveVerTag = function(verObj, tag) {
  if (tag == null) {
    tag = 'latest';
  }
  if (!verObj) {
    return '';
  }
  return verObj[tag] || '';
};
module.exports = function(deps, useCache, mirror, expire, wantedTag) {
  if (useCache == null) {
    useCache = true;
  }
  if (expire == null) {
    expire = EXPIRE;
  }
  if (wantedTag == null) {
    wantedTag = 'latest';
  }
  return Promise.all(deps.map(function(dep) {
    var name, promise, tag, verObj;
    name = typeof dep === 'object' ? dep.packageName : dep;
    verObj = cache.get(name, expire);
    tag = (typeof dep === 'object' && dep.tryTag) ? dep.declareVer : wantedTag;
    promise = verObj && useCache ? Promise.resolve(resolveVerTag(verObj, tag)) : latestVer(name, mirror).then(function(verObj) {
      cache.set(name, verObj);
      return resolveVerTag(verObj, tag);
    });
    if (typeof dep === 'object') {
      return promise.then(function(ver) {
        if (ver) {
          dep.newVer = ver;
          dep.installName = dep.tryTag ? tag : ver;
          if (dep.tryTag) {
            dep.declareVer = ver;
          }
          strategy.version(dep);
        } else {
          dep.installName = dep.newVer = dep.installedVer;
          dep.needUpdate = false;
        }
        return dep;
      });
    } else {
      return promise;
    }
  })).then(function(deps) {
    cache.record();
    return deps;
  });
};
