"use strict";
var Promise, _, chalk, checkVer, formatPackages, fs, getToWrite, globalPackage, npm, npmUp, npmUpGlobal, npmUpSubDir, option, packageMgr, parseOpts, parsePackage, path, prepare, ref, semver, spinner, util,
  indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; };
chalk = require('chalk');
ref = fs = require('nofs'), path = ref.path, Promise = ref.Promise;
_ = require('underscore');
semver = require('semver');
spinner = require('./spinner');
npm = require('./npm');
util = require('./util');
checkVer = require('./checkVersion');
packageMgr = require('./package');
option = {};
globalPackage = {};
parseOpts = function(opts) {
  if (opts == null) {
    opts = {};
  }
  _.defaults(opts, {
    include: "",
    exclude: [],
    writeBack: false,
    install: false,
    lock: false,
    all: false,
    dev: true,
    dep: true,
    optional: true,
    silent: false,
    lockAll: false,
    cache: true,
    logLevel: 'error',
    cwd: process.cwd(),
    warning: true,
    mirror: '',
    tag: 'latest',
    excludeLocked: false
  });
  opts.mirror = util.getRegistry(opts.mirror);
  opts.all && _.assign(opts, {
    writeBack: true,
    install: true,
    lock: true
  });
  opts.exclude = _.compact(opts.exclude);
  opts.include && (opts.include = _.compact(opts.include));
  return opts;
};
parsePackage = function(name, ver, type) {
  var declareVer, installedVer, res;
  if (Array.isArray(option.include) && !(indexOf.call(option.include, name) >= 0)) {
    return null;
  }
  if (indexOf.call(option.exclude, name) >= 0) {
    return null;
  }
  if (option.excludeLocked && semver.valid(ver)) {
    return null;
  }
  if (ver.indexOf('//') > 0) {
    return null;
  }
  res = {};
  if (type === 'g') {
    declareVer = installedVer = ver;
  } else {
    declareVer = semver.validRange(ver) ? ver.trim() : null;
    declareVer === '' && (declareVer = '*');
    if (!declareVer) {
      res.declareVer = ver;
      res.tryTag = true;
    }
    installedVer = packageMgr.getPackageVersion(name);
  }
  return _.assign({
    packageName: name,
    declareVer: declareVer,
    installedVer: installedVer,
    baseVer: installedVer,
    newVer: '',
    installName: '',
    type: type,
    needUpdate: false,
    warnMsg: '',
    tryTag: false
  }, res);
};
formatPackages = function(obj, type) {
  return _.map(obj, function(version, name) {
    return parsePackage(name, version, type);
  });
};
prepare = function() {
  var deps, e;
  try {
    globalPackage = packageMgr.readPackageFile();
  } catch (error) {
    e = error;
    throw new Error((e instanceof SyntaxError ? 'unvalid package.json!' : 'package.json Not Found!'));
  }
  deps = [];
  if (option.dep) {
    deps = deps.concat(formatPackages(globalPackage.dependencies, 'S'));
  }
  if (option.dev) {
    deps = deps.concat(formatPackages(globalPackage.devDependencies, 'D'));
  }
  if (option.optional) {
    deps = deps.concat(formatPackages(globalPackage.optionalDependencies, 'O'));
  }
  return deps = _.compact(deps);
};
getToWrite = function(arg, arg1) {
  var declareVer, lock, lockAll, m, newVer, prefix, tryTag;
  declareVer = arg.declareVer, newVer = arg.newVer, tryTag = arg.tryTag;
  lock = arg1.lock, lockAll = arg1.lockAll;
  if (declareVer === '*' || declareVer === '') {
    if (lockAll) {
      return newVer;
    } else {
      return '*';
    }
  }
  if (tryTag) {
    return null;
  }
  prefix = lock || semver.valid(declareVer) ? '' : (m = declareVer.match(/^([><=\s^~]+)[\d\w.-\s]*$/)) ? m[1] : '^';
  return prefix + newVer;
};
npmUp = function() {
  var deps, e;
  process.chdir(option.cwd);
  try {
    deps = prepare();
  } catch (error) {
    e = error;
    console.error(chalk.red(util.errorSign + (" " + e)));
    return Promise.reject();
  }
  spinner.start();
  return checkVer(deps, option.cache, option.mirror, void 0, option.tag).then(function(newDeps) {
    var chain, install, toUpdate;
    spinner.stop();
    deps = newDeps;
    util.print(deps, option.warning);
    toUpdate = deps.filter(function(dep) {
      return dep.needUpdate && dep.installedVer;
    }).map(function(dep) {
      return dep.packageName + "@" + dep.installName;
    });
    chain = Promise.resolve();
    if (toUpdate.length === 0) {
      util.logSucc("Everything is new!");
    }
    if (option.writeBack) {
      chain = chain.then(function() {
        return deps.forEach(function(dep) {
          var toWrite;
          toWrite = getToWrite(dep, option);
          if (!toWrite) {
            return;
          }
          switch (dep.type) {
            case 'S':
              return globalPackage.dependencies[dep.packageName] = toWrite;
            case 'D':
              return globalPackage.devDependencies[dep.packageName] = toWrite;
            case 'O':
              return globalPackage.optionalDependencies[dep.packageName] = toWrite;
          }
        });
      }).then(function() {
        var packageFile;
        ['dependencies', 'devDependencies', 'optionalDependencies'].forEach(function(k) {
          if (_.isEmpty(globalPackage[k])) {
            return delete globalPackage[k];
          }
        });
        packageFile = path.join(process.cwd(), 'package.json');
        return fs.outputJSON(packageFile, globalPackage, {
          space: 2
        });
      }).then(function() {
        return util.logSucc("package.json has been updated!");
      });
    }
    if (option.install) {
      install = require('./install');
      chain = chain.then(function() {
        return install(toUpdate, option.cwd);
      });
    }
    return chain;
  });
};
npmUpSubDir = function() {
  process.chdir(option.cwd);
  return fs.glob('*/package.json').then(function(packs) {
    return packs.reduce(function(prev, cur) {
      var cwd, dir;
      cwd = process.cwd();
      dir = path.dirname(cur);
      return prev.then(function() {
        console.log('\n' + chalk.magenta.bold(util.circleSign, chalk.underline(path.basename(dir))));
        option.cwd = path.join(cwd, dir);
        return npmUp();
      })["catch"](function() {});
    }, Promise.resolve());
  });
};
npmUpGlobal = function() {
  if (option.install && !util.checkPrivilege()) {
    console.error(chalk.red(util.errorSign + " Permission Denied"));
    console.error(chalk.yellow("Please try running this command again as root/Administrator"));
    process.exit(1);
  }
  util.logInfo('Searching global packages...');
  return util.allPackagesIn(npm.globalDir).then(function(packages) {
    var deps, globalDep;
    globalDep = packages.reduce(function(obj, pack) {
      obj[pack.name] = pack.version;
      return obj;
    }, {});
    console.log(chalk.cyan(Object.keys(globalDep).sort().join(' ')));
    deps = _.map(globalDep, function(val, key) {
      return parsePackage(key, val, 'g');
    });
    spinner.start();
    return checkVer(_.compact(deps), option.cache, option.mirror, void 0, option.tag);
  }).then(function(newDeps) {
    var deps, oldLen, toUpdate;
    spinner.stop();
    console.log('');
    deps = newDeps;
    util.print(deps, option.warning);
    toUpdate = deps.filter(function(dep) {
      return dep.needUpdate && dep.installedVer;
    }).map(function(dep) {
      return dep.packageName + "@" + dep.newVer;
    });
    if (toUpdate.length === 0) {
      util.logSucc("Everything is new!");
      return Promise.resolve();
    } else if (option.install) {
      oldLen = toUpdate.length;
      toUpdate = toUpdate.filter(function(name) {
        return name.indexOf('npm@') !== 0;
      });
      if (oldLen !== toUpdate.length) {
        util.logWarn(chalk.yellow("It may cause a broken error to install npm by npm-up sometimes.", "Please use", chalk.cyan("[sudo] npm i npm -g"), "instead."));
        console.log(chalk.green("   If you know the reason, please put forward an issue."));
      }
      return require('./install')(toUpdate);
    }
  });
};
module.exports = function(opt, checkUpdate) {
  var npmOpt;
  if (checkUpdate == null) {
    checkUpdate = false;
  }
  option = parseOpts(opt);
  npmOpt = {
    loglevel: option.logLevel,
    global: !!opt.global,
    save: false
  };
  if (option.mirror) {
    npmOpt.registry = option.mirror;
  }
  return new Promise(function(resolve, reject) {
    return npm.load(function() {
      var key, p, promise, val;
      for (key in npmOpt) {
        val = npmOpt[key];
        npm.config.set(key, val);
      }
      option.mirror = npm.config.get('registry').slice(0, -1);
      if (option.mirror.indexOf('.npmjs.org') < 0) {
        util.logWarn("Please ensure that the mirror is in sync with " + (util.getRegistry("npm")));
      }
      promise = opt.global ? npmUpGlobal() : opt.All ? npmUpSubDir() : npmUp();
      if (checkUpdate) {
        p = require('./updateSelf')(option.mirror);
        return promise.then(function() {
          return p.log();
        });
      }
    });
  });
};
