"use strict";
var Promise, _, chalk, circleSign, cwdFilePath, debug, errorSign, fs, host, infoSign, isWin, logInfo, logSucc, logWarn, okSign, padLeft, padRight, path, protocolReg, ref, repeat, warnSign,
  slice = [].slice;
ref = fs = require('nofs'), path = ref.path, Promise = ref.Promise;
chalk = require('chalk');
_ = require('underscore');
isWin = process.platform === 'win32';
warnSign = isWin ? '‼ ' : '⚠  ';
errorSign = isWin ? '× ' : '✖  ';
okSign = isWin ? '√  ' : '✔  ';
infoSign = isWin ? 'i ' : 'ℹ ';
circleSign = isWin ? '*' : '●';
cwdFilePath = function() {
  var names;
  names = 1 <= arguments.length ? slice.call(arguments, 0) : [];
  return path.join.apply(path, [process.cwd()].concat(names));
};
logInfo = function(str) {
  return console.log(chalk.bold(chalk.yellow('\n>> ') + chalk.green(str)));
};
logSucc = function(str) {
  return console.log(chalk.green.bold("\n" + okSign + str));
};
logWarn = function(str) {
  return console.log(' ', chalk.grey(chalk.yellow(warnSign) + str));
};
host = {
  npm: 'http://registry.npmjs.org',
  taobao: 'http://registry.npm.taobao.org',
  cnpmjs: 'http://registry.cnpmjs.org'
};
protocolReg = /^https?:\/\//;
repeat = function(str, n) {
  var i, j, ref1, res;
  n = parseInt(n);
  res = '';
  if (n < 1) {
    return res;
  }
  for (i = j = 0, ref1 = n; 0 <= ref1 ? j <= ref1 : j >= ref1; i = 0 <= ref1 ? ++j : --j) {
    res += str;
  }
  return res;
};
padRight = function(str, n) {
  var strLen;
  str += '';
  strLen = str.length;
  if (strLen >= n) {
    return str;
  } else {
    return str + repeat(' ', n - strLen);
  }
};
padLeft = function(str, n) {
  var strLen;
  str += '';
  strLen = str.length;
  if (strLen >= n) {
    return str;
  } else {
    return repeat(' ', n - strLen) + str;
  }
};
debug = function() {
  var ref1;
  if ((ref1 = process.env.DEBUG) === true || ref1 === 'on' || ref1 === 'true') {
    return console.log.apply(console, arguments);
  }
};
module.exports = {
  debug: debug,
  cwdFilePath: cwdFilePath,
  errorSign: errorSign,
  warnSign: warnSign,
  okSign: okSign,
  circleSign: circleSign,
  logInfo: logInfo,
  logSucc: logSucc,
  logWarn: logWarn,
  getRegistry: function(name) {
    name = name.trim();
    if (name === '') {
      return '';
    } else if (host[name]) {
      return host[name];
    } else if (protocolReg.test(name)) {
      return name;
    } else {
      return 'http://' + name;
    }
  },
  print: function(deps, showWarn) {
    var depsObj, results, type;
    if (showWarn == null) {
      showWarn = true;
    }
    depsObj = _.groupBy(deps, 'type');
    results = [];
    for (type in depsObj) {
      deps = depsObj[type];
      if ((deps.filter(function(dep) {
        return dep.needUpdate || (showWarn && dep.warnMsg);
      })).length) {
        console.log(chalk.bold.white((function() {
          switch (type) {
            case 'S':
              return 'dependencies';
            case 'D':
              return 'devDependencies';
            case 'O':
              return 'optionalDependencies';
            case 'g':
              return 'Global Dependencies';
          }
        })()));
      }
      results.push(deps.map(function(dep) {
        dep.needUpdate && console.log('  ', chalk.green(circleSign), chalk.cyan.bold(padRight(dep.packageName, 40)), chalk.red(padLeft(dep.baseVer.toString(), 8)), '→', chalk.green(dep.newVer.toString()));
        return showWarn && dep.warnMsg && logWarn("" + dep.warnMsg);
      }));
    }
    return results;
  },
  curVer: (function() {
    return require('../package.json').version;
  })(),
  checkPrivilege: function() {
    var errno;
    try {
      fs.linkSync(__filename, path.join(__dirname, 'linkTest.temp'));
      fs.removeSync(path.join(__dirname, '*.temp'));
      return true;
    } catch (error) {
      errno = error.errno;
      return errno !== -13;
    }
  },
  allPackagesIn: function(dir) {
    return fs.glob(path.join(dir, '*/package.json')).then(function(files) {
      return files.map(function(f) {
        try {
          return require(f);
        } catch (error) {
          return null;
        }
      }).filter(function(f) {
        return !!f;
      });
    });
  },
  isNPMRegistry: function(url) {
    return url.indexOf('.npmjs.org') > 0;
  },
  promisify: fs.PromiseUtils.promisify
};
